@@ -1,4 +1,5 @@
-""""Waylay Python SDK"""
+"""Waylay Python SDK."""
+
 from .client import WaylayClient
 from .config import WaylayConfig
 from .auth import CredentialsType, ClientCredentials
