@@ -35,6 +35,8 @@ Settings = MutableMapping[str, str]
 
 DEFAULT_PROFILE = '_default_'
 SERVICE_KEY_API = 'waylay_api'
+DEFAULT_DOC_URL: str = 'https://docs.waylay.io/#'
+DOC_URL_KEY: str = 'doc_url'
 
 
 class WaylayConfig():
@@ -47,9 +49,6 @@ class WaylayConfig():
 
     _token_auth_provider: Type[WaylayTokenAuth] = WaylayTokenAuth
 
-    DOC_URL: str = 'https://docs.waylay.io'
-    IODOC_URL: str = 'https://docs-io.waylay.io/#'
-
     def __init__(
         self, credentials: WaylayCredentials = None, profile: str = DEFAULT_PROFILE,
         settings: TenantSettings = None, fetch_tenant_settings=True,
@@ -105,6 +104,11 @@ class WaylayConfig():
         """Get the accounts url."""
         return _root_url_for(self.credentials.accounts_url)
 
+    @property
+    def doc_url(self) -> str:
+        """Get the root url of the documentation site."""
+        return self.settings.get(DOC_URL_KEY, DEFAULT_DOC_URL)
+
     @property
     def tenant_settings(self) -> TenantSettings:
         """Get the tenant settings as stored on accounts.
