@@ -1,5 +1,8 @@
 """Base exception class hierarchy for errors in the waylay client."""
-from simple_rest_client.exceptions import ErrorWithResponse as _ErrorWithResponse
+from simple_rest_client.exceptions import (
+    ErrorWithResponse as _ErrorWithResponse,
+    ClientConnectionError as _ClientConnectionError
+)
 
 
 class WaylayError(Exception):
@@ -21,6 +24,15 @@ class RequestError(WaylayError):
     """
 
 
+class RestConnectionError(RequestError):
+    """Exception class for request errors that are caused by client connection errors."""
+
+    @classmethod
+    def from_cause(cls, cause: _ClientConnectionError):
+        """Convert a client connection error to a waylay client error."""
+        return cls(cause)
+
+
 class RestError(WaylayError):
     """Exception class for failures to make a REST call."""
 
