@@ -77,7 +77,7 @@ class WaylayAction(dict):
         return self.get('description')
 
     @property
-    def hal_links(self) -> Dict[str, Dict[str, str]]:
+    def doc_links(self) -> Dict[str, Dict[str, str]]:
         """Get (documentation) links templates."""
         roots = self.resource.hal_roots
         return {
@@ -142,10 +142,18 @@ class WaylayResource(Resource):
     def decorate_action(self, action_name):
         """Decorate the action if a 'decorators' definition exist."""
         action = self.get_action(action_name)
+        action_attrs = dict(
+            resource_name=self.resource_name,
+            action_name=action_name
+        )
         decorators = action.get('decorators', None)
+        action_method = getattr(self, action_name)
+        action_method.__dict__.update(action_attrs)
         if decorators:
             for decorator in decorators:
-                setattr(self, action_name, decorator(getattr(self, action_name)))
+                action_method = decorator(action_method)
+                action_method.__dict__.update(action_attrs)
+        setattr(self, action_name, action_method)
 
     def __repr__(self):
         """Get a technical string representation of this action."""
@@ -167,13 +175,12 @@ class WaylayResource(Resource):
                 href_root
             ).safe_substitute(
                 root_url=self.api_root_url,
-                doc_url=self.service.config.DOC_URL,
-                iodoc_url=self.service.config.IODOC_URL,
+                doc_url=self.service.config.doc_url,
             )
             for link, href_root in self.link_roots.items()
         }
 
-    def hal_links(self, action: Optional[str]) -> Dict[str, Dict[str, str]]:
+    def doc_links(self, action: Optional[str]) -> Dict[str, Dict[str, str]]:
         """Create a HAL `_links` representation for (documentation) links.
 
         Attributes:
@@ -189,8 +196,7 @@ class WaylayResource(Resource):
                     href_root + hrefs.get(link_rel, '')
                 ).safe_substitute(
                     root_url=self.api_root_url,
-                    doc_url=self.service.config.DOC_URL,
-                    iodoc_url=self.service.config.IODOC_URL,
+                    doc_url=self.service.config.doc_url,
                 )
             )
             for link_rel, href_root in self.link_roots.items()
@@ -302,13 +308,12 @@ class WaylayService(API):
             ")>"
         )
 
-    def hal_links(self) -> Dict[str, Dict[str, str]]:
+    def doc_links(self) -> Dict[str, Dict[str, str]]:
         """Create a HAL `_links` representation for the documentation links."""
         return {
             rel: dict(href=Template(href).safe_substitute(
                 root_url=self.root_url,
-                doc_url=self.config.DOC_URL,
-                iodoc_url=self.config.IODOC_URL,
+                doc_url=self.config.doc_url,
             ))
             for rel, href in self.link_templates.items()
         }
