@@ -1,11 +1,14 @@
 """Resource action method decorators that are generally usefull."""
 
+import time
 from functools import wraps
 from typing import (
     Union, List, Mapping, Any, Callable, TypeVar, Dict, Optional
 )
-from simple_rest_client.exceptions import ErrorWithResponse
-from ..exceptions import RestResponseError, RestResponseParseError
+
+from logging import Logger, getLevelName, INFO
+from simple_rest_client.exceptions import ErrorWithResponse, ClientConnectionError
+from ..exceptions import RestResponseError, RestResponseParseError, RestConnectionError
 
 
 def suppress_header_decorator(header_key):
@@ -31,6 +34,8 @@ def exception_decorator(action_method):
             return action_method(*args, **kwargs)
         except ErrorWithResponse as exc:
             raise RestResponseError.from_cause(exc) from exc
+        except ClientConnectionError as exc:
+            raise RestConnectionError.from_cause(exc) from exc
     return wrapped
 
 
@@ -38,13 +43,13 @@ def default_params_decorator(default_params: Dict[str, Any]):
     """Create a decorator that initializes default url request parameters."""
     def decorator(action_method):
         @wraps(action_method)
-        def wrapped(slf, *args, **kwargs):
+        def wrapped(*args, **kwargs):
             params = kwargs.pop('params', {})
             for key, value in default_params.items():
                 if key not in params:
                     params[key] = value
             kwargs['params'] = params
-            return action_method(slf, *args, **kwargs)
+            return action_method(*args, **kwargs)
         wrapped.__doc__ = (
             (wrapped.__doc__ or '') +
             "Uses default url parameters (`params` argument):" +
@@ -57,6 +62,47 @@ def default_params_decorator(default_params: Dict[str, Any]):
     return decorator
 
 
+def default_header_decorator(default_headers: Dict[str, Any]):
+    """Create a decorator that initializes default header parameters."""
+    def decorator(action_method):
+        @wraps(action_method)
+        def wrapped(*args, **kwargs):
+            headers = kwargs.pop('headers', {})
+            for key, value in default_headers.items():
+                if key not in headers:
+                    headers[key] = str(value)
+            kwargs['headers'] = headers
+            return action_method(*args, **kwargs)
+        wrapped.__doc__ = (
+            (wrapped.__doc__ or '') +
+            "Uses default headers (`headers` argument):" +
+            "".join(
+                f"\n    '{key}': {default_value}"
+                for key, default_value in default_headers.items()
+            )
+        )
+        return wrapped
+    return decorator
+
+
+def default_timeout_decorator(default_timeout: float):
+    """Create a decorator that modifies the default http client timeout."""
+
+    def decorator(action_method):
+        @wraps(action_method)
+        def wrapped(*args, **kwargs):
+            timeout = kwargs.pop('timeout', default_timeout)
+            return action_method(*args, timeout=timeout, **kwargs)
+        timeout_arg = f'timeout={default_timeout}'
+        wrapped.__doc__ = (
+            (wrapped.__doc__ or '') + f"""
+Uses a modified default http client timeout:
+    {timeout_arg:30}: Client timeout for the REST request.
+""")
+        return wrapped
+    return decorator
+
+
 T = TypeVar('T')
 
 
@@ -158,3 +204,40 @@ def _select_path_from(
 
     ctx_path_str = '.'.join(str(p) for p in new_ctx_path)
     raise AttributeError(f"Cannot find '{ctx_path_str}' in response body.")
+
+
+def log_server_timing_decorator(logger: Optional[Logger] = None, level: int = INFO):
+    """Create a decorator that inspects and logs a 'Server-Timing' response header.
+
+    This decorator should be below any response or exception decorator.
+    """
+    def decorator(action_method):
+        logger.debug(f'adding Server-Timing logging for {action_method} at {getLevelName(level)}', action_method, )
+
+        def _log_server_timing(server_timing, start):
+            if server_timing is None:
+                end = time.time()
+                logger.log(level, 'no-server-timing-available;dur=%.3f', (end-start) / 1000)
+                return
+            for line in server_timing.split(','):
+                logger.log(level, line.strip())
+
+        @wraps(action_method)
+        def wrapped(*args, **kwargs):
+            start = time.time()
+            try:
+                response = action_method(*args, **kwargs)
+                _log_server_timing(response.headers.get('Server-Timing', None), start)
+                return response
+            except ErrorWithResponse as exc:
+                _log_server_timing(exc.response.headers.get('Server-Timing', None), start)
+                raise exc
+            except Exception:
+                _log_server_timing(None, start)
+                raise
+        wrapped.__doc__ = (
+            (wrapped.__doc__ or '') + f"""
+Logs server timing on {logger.name} at level {getLevelName(level)}
+""")
+        return wrapped
+    return decorator
