@@ -1,3 +1,4 @@
-"""Analytics REST service"""
+"""Analytics REST service."""
+
 from ._service import AnalyticsService
 from ._decorators import MultiFrameHandling
