@@ -6,8 +6,9 @@ from typing import (
     Union, List
 )
 import pandas as pd
-from simple_rest_client.exceptions import ErrorWithResponse
+from simple_rest_client.exceptions import ErrorWithResponse, ClientConnectionError
 
+from waylay.exceptions import RestConnectionError
 from ._util import (
     parse_df,
     RENDER_MODE_COMPACT_WS
@@ -28,6 +29,8 @@ def analytics_exception_decorator(action_method):
             return action_method(*args, **kwargs)
         except ErrorWithResponse as exc:
             raise AnalyticsActionError.from_cause(exc) from exc
+        except ClientConnectionError as exc:
+            raise RestConnectionError.from_cause(exc) from exc
     return wrapped
 
 
