@@ -40,7 +40,6 @@ class QueryResource(WaylayResource):
 
     link_roots = {
         'doc': '${doc_url}/api/query/',
-        'iodoc': '${iodoc_url}/api/query/',
         'apidoc':  '${root_url}/apidocs/'
     }
 
@@ -51,8 +50,7 @@ class QueryResource(WaylayResource):
             'decorators': CONFIG_LIST_DECORATORS,
             'description': 'List or search <em>Query Configurations</em>.',
             'links': {
-                'doc': 'search/#search',
-                'iodoc': '?id=data-query-search-api',
+                'doc': '?id=data-query-search-api',
                 'apidoc': '#/query%20config/get_config_query'
             }
         },
@@ -62,8 +60,7 @@ class QueryResource(WaylayResource):
             'decorators': CONFIG_ENTITY_DECORATORS,
             'description': 'Create a new <em>Query Configuration</em>.',
             'links': {
-                'doc': 'crud/#create',
-                'iodoc': '?id=create',
+                'doc': '?id=create',
                 'apidoc': '#/query%20config/post_config_query'
             }
         },
@@ -73,8 +70,7 @@ class QueryResource(WaylayResource):
             'decorators': CONFIG_ENTITY_DECORATORS,
             'description': 'Fetch the named <em>Query Configuration</em>.',
             'links': {
-                'doc': 'crud/#retrieve',
-                'iodoc': '?id=retrieve',
+                'doc': '?id=retrieve',
                 'apidoc': '#/query%20config/get_config_query__query_name_'
             }
         },
@@ -84,8 +80,7 @@ class QueryResource(WaylayResource):
             'decorators': CONFIG_STATUS_DECORATORS,
             'description': 'Remove the named <em>Query Configuration</em>.',
             'links': {
-                'doc': 'crud/#delete',
-                'iodoc': '?id=delete',
+                'doc': '?id=delete',
                 'apidoc': '#/query%20config/delete_config_query__query_name_'
             }
         },
@@ -95,8 +90,7 @@ class QueryResource(WaylayResource):
             'decorators': CONFIG_ENTITY_DECORATORS,
             'description': 'Replace the named <em>Query Configuration</em>.',
             'links': {
-                'doc': 'crud/#replace',
-                'iodoc': '?id=replace',
+                'doc': '?id=replace',
                 'apidoc': '#/query%20config/put_config_query__query_name_'
             }
         },
@@ -108,8 +102,7 @@ class QueryResource(WaylayResource):
                 'Execute the timeseries query specified by the named <em>Query Configuration</em>.'
             ),
             'links': {
-                'doc': 'data/#query-execution',
-                'iodoc': '?id=query-execution',
+                'doc': '?id=query-execution',
                 'apidoc': '#/data/get_data_query__query_name_'
             }
         },
@@ -119,8 +112,7 @@ class QueryResource(WaylayResource):
             'decorators': DATA_RESPONSE_DECORATORS,
             'description': 'Execute the timeseries query specified in the request body.',
             'links': {
-                'doc': 'data/#query-execution',
-                'iodoc': '?id=query-execution',
+                'doc': '?id=query-execution',
                 'apidoc': '#/data/post_data_query'
             }
         },
