@@ -1,6 +1,11 @@
 """Exceptions specific to the Byoml Service."""
 
-from ...exceptions import RestResponseError, RestResponseParseError, RequestError
+from ...exceptions import (
+    RestResponseError,
+    RestResponseParseError,
+    RequestError,
+    RestError
+)
 
 
 class ByomlActionError(RestResponseError):
@@ -18,3 +23,7 @@ class ByomlActionParseError(RestResponseParseError, ByomlActionError):
 
 class ByomlValidationError(RequestError):
     """Exception class for BYOML validation errors."""
+
+
+class ModelNotReadyError(RestError):
+    """Indicates that a byoml action should be retried."""
