@@ -4,6 +4,8 @@
 from .._base import WaylayService
 
 from .model import ModelResource
+from .runtime import RuntimeResource
+from .framework import FrameworkResource
 from .about import AboutResource
 
 
@@ -16,8 +18,12 @@ class ByomlService(WaylayService):
 
     resource_definitions = {
         'model': ModelResource,
-        'about': AboutResource
+        'about': AboutResource,
+        'framework': FrameworkResource,
+        'runtime': RuntimeResource
     }
 
     model: ModelResource
+    runtime: RuntimeResource
+    framework: FrameworkResource
     about: AboutResource
