@@ -1,33 +1,47 @@
 """REST definitions for the 'model' entity of the 'byoml' service."""
 
-from zipfile import ZipFile
-from io import BytesIO
-from functools import wraps
-from typing import Callable, List, Any, Union, Tuple, Dict, Optional
-import os
-import tempfile
+from contextlib import contextmanager
 
-import joblib
+from functools import wraps
+from typing import (
+    Callable, List, Any, Union, Dict, Optional
+)
+import json
 import pandas as pd
 import numpy as np
+import logging
 
 from .._base import WaylayResource
 from .._decorators import (
     return_body_decorator,
     return_path_decorator,
-    suppress_header_decorator
+    default_timeout_decorator,
+    suppress_header_decorator,
+    log_server_timing_decorator,
 )
 from ._decorators import (
     byoml_exception_decorator,
-    byoml_retry_decorator
+    byoml_retry_decorator,
+    byoml_raise_not_ready_get,
+    byoml_retry_upload_after_deletion_decorator,
+    DEFAULT_BYOML_MODEL_TIMEOUT
 )
 from ...exceptions import (
     RestRequestError,
 )
-from ._exceptions import (
-    ByomlValidationError,
+from ._model_archive import (
+    ModelPlugArchiveBuilder,
+    ModelZipArchiveBuilder,
+    ByomlModel, PathLike
 )
 
+LOG = logging.getLogger(__name__)
+
+DEFAULT_MODEL_UPLOAD_TIMEOUT = 120
+
+LOG_SERVER_TIMING = logging.getLogger(__name__ + '.server-timing')
+byoml_server_timing_decorator = log_server_timing_decorator(LOG_SERVER_TIMING, logging.INFO)
+
 
 def _input_data_as_list(input_data):
     if isinstance(input_data, list):
@@ -66,7 +80,7 @@ def model_execution_request_decorator(action_method):
     as `instances` in the request body.
     """
     @wraps(action_method)
-    def wrapped(model_name, input_data, **kwargs):
+    def wrapped(model_name, input_data=None, **kwargs):
         body = kwargs.pop('body', {})
         if 'instances' not in body:
             body = {
@@ -81,24 +95,11 @@ def model_execution_request_decorator(action_method):
     return wrapped
 
 
-DEFAULT_DECORATORS = [
-    byoml_exception_decorator,
-    return_body_decorator,
-]
-
-
-def _get_model_decorator() -> List[Callable]:
-    return [
-        byoml_retry_decorator,
-        byoml_exception_decorator,
-        return_body_decorator,
-    ]
-
-
 def _execute_model_decorators(response_key: str) -> List[Callable]:
     return [
-        byoml_retry_decorator,
+        byoml_server_timing_decorator,
         byoml_exception_decorator,
+        byoml_retry_decorator,
         model_execution_request_decorator,
         return_path_decorator(
             [response_key],
@@ -107,73 +108,11 @@ def _execute_model_decorators(response_key: str) -> List[Callable]:
     ]
 
 
-DEFAULT_BYOML_MODEL_TIMEOUT = 60
-
-# type aliases for documentation purpose
-PathLike = Union[str, os.PathLike]
-PytorchModel = Any
-TensorflowModel = Any
-XgboostModel = Any
-SklearnModel = Any
-ByomlModel = Union[PytorchModel, TensorflowModel, XgboostModel, SklearnModel]
-ModelSerializer = Callable[[PathLike, ByomlModel], PathLike]
-
-
-def _serialize_torch(work_dir: PathLike, trained_model: PytorchModel) -> PathLike:
-    """Serialize a pytorch model to a `model.pt` file."""
-    # assuming a TorchScript model
-    import torch  # pylint: disable=import-error
-
-    model_file = f'{work_dir}/model.pt'
-    model_script = torch.jit.script(trained_model)
-    model_script.save(model_file)
-    return model_file
-
-
-def _serialize_joblib(work_dir: PathLike, trained_model: SklearnModel) -> PathLike:
-    """Serialize a sklearn model to a `model.joblib` file."""
-    model_file = f'{work_dir}/model.joblib'
-    joblib.dump(trained_model, model_file)
-    return model_file
-
-
-def _serialize_tf(work_dir: PathLike, trained_model: TensorflowModel) -> PathLike:
-    """Serialize a tensorflow model to a model folder."""
-    import tensorflow as tf  # pylint: disable=import-error
-
-    tf.saved_model.save(trained_model, work_dir)
-    return work_dir
-
-
-def _serialize_bst(work_dir: PathLike, trained_model: XgboostModel) -> PathLike:
-    """Serialize a xgboost model to a `model.bst` file."""
-    if hasattr(trained_model, 'save_model'):
-        model_file = f'{work_dir}/model.bst'
-        trained_model.save_model(model_file)
-        return model_file
-    raise ByomlValidationError('Could not serialise this model: missing `save_model` method.')
-
-
-SUPPORTED_FRAMEWORKS: Dict[str, ModelSerializer] = {
-    "pytorch": _serialize_torch,
-    "sklearn": _serialize_joblib,
-    "tensorflow": _serialize_tf,
-    "xgboost": _serialize_bst,
-}
-
-
-def assert_dir_exists(dir_name: PathLike):
-    """Raise error if the input directory does not exist."""
-    if not os.path.exists(str(dir_name)):
-        raise ByomlValidationError(f"The directory '{dir_name}' does not exist.")
-
-
 class ModelResource(WaylayResource):
     """REST Resource for the 'model' entity of the 'byoml' service."""
 
     link_roots = {
-        'doc': '${doc_url}/api/byoml/',
-        'iodoc': '${iodoc_url}/api/byoml/?id='
+        'doc': '${doc_url}/api/byoml/?id='
     }
 
     actions = {
@@ -181,85 +120,120 @@ class ModelResource(WaylayResource):
             'method': 'GET',
             'url': '/models',
             'decorators': [
+                byoml_server_timing_decorator,
                 byoml_exception_decorator,
                 return_path_decorator(['available_models'])
             ],
             'description': 'List the metadata of the deployed <em>BYOML Models</em>',
             'links': {
-                'doc': '#overview-of-the-api',
-                'iodoc': 'overview-of-the-api'
+                'doc': 'overview-of-the-api'
             },
         },
         'list_names': {
             'method': 'GET',
             'url': '/models',
             'decorators': [
+                byoml_server_timing_decorator,
                 byoml_exception_decorator,
                 return_path_decorator(['available_models', 'name'])
             ],
             'description': 'List the names of deployed <em>BYOML Models</em>',
             'links': {
-                'doc': '#overview-of-the-api',
-                'iodoc': 'overview-of-the-api'
+                'doc': 'overview-of-the-api'
             },
         },
         '_create': {
             'method': 'POST',
             'url': '/models',
-            'decorators': DEFAULT_DECORATORS,
+            'decorators': [
+                byoml_server_timing_decorator,
+                byoml_exception_decorator,
+                default_timeout_decorator(DEFAULT_MODEL_UPLOAD_TIMEOUT),
+                byoml_retry_upload_after_deletion_decorator,
+                return_body_decorator,
+            ],
             'description': (
                 'Build and create a new <em>BYOML Model</em> as specified in the request'
             ),
             'name': 'upload',
             'links': {
-                'doc': '#how-to-upload-your-model',
-                'iodoc': 'how-to-upload-your-model'
+                'doc': 'how-to-upload-your-model'
             },
         },
         '_replace': {
             'method': 'PUT',
             'url': '/models/{}',
             'name': 'replace',
-            'decorators': DEFAULT_DECORATORS,
+            'decorators': [
+                byoml_server_timing_decorator,
+                byoml_exception_decorator,
+                default_timeout_decorator(DEFAULT_MODEL_UPLOAD_TIMEOUT),
+                return_body_decorator,
+            ],
             'description': 'Build and replace the named <em>BYOML Model</em>',
             'links': {
-                'doc': '#overwriting-a-model',
-                'iodoc': 'overwriting-a-model'
+                'doc': 'overwriting-a-model'
             },
         },
         'get': {
             'method': 'GET',
             'url': '/models/{}',
-            'decorators': _get_model_decorator(),
+            'decorators': [
+                byoml_server_timing_decorator,
+                byoml_exception_decorator,
+                byoml_raise_not_ready_get,
+                byoml_retry_decorator,
+                return_body_decorator,
+            ],
             'description': 'Fetch the metadata of the named <em>BYOML Model</em>',
             'links': {
-                'doc': '#checking-out-your-model',
-                'iodoc': 'checking-out-your-model'
+                'doc': 'checking-out-your-model'
+            },
+        },
+        'update': {
+            'method': 'PATCH',
+            'url': '/models/{}',
+            'decorators': [
+                byoml_server_timing_decorator,
+                byoml_exception_decorator,
+                byoml_retry_decorator,
+                return_body_decorator,
+            ],
+            'description': (
+                'Update the metadata of the named <em>BYOML Model</em>.\n'
+                'Only metadata attributes can be modified.\n'
+                '    model.update("my_model", body={"description":"Updated"})'
+            ),
+            'links': {
+                'doc': 'update-metadata-for-a-model'
             },
         },
         'get_content': {
             'method': 'GET',
             'url': '/models/{}/content',
-            'decorators': [byoml_exception_decorator],
+            'decorators': [
+                byoml_server_timing_decorator,
+                byoml_exception_decorator
+            ],
             'description': 'Fetch the content of the named <em>BYOML Model</em>',
             'links': {
-                'doc': '#checking-out-your-model',
-                'iodoc': 'checking-out-your-model'
+                'doc': 'checking-out-your-model'
             },
         },
         'examples': {
             'method': 'GET',
             'url': '/models/{}/examples',
             'decorators': [
+                byoml_server_timing_decorator,
                 byoml_exception_decorator,
+                byoml_retry_decorator,
                 return_path_decorator(['example_payloads'])
             ],
             'description': (
                 'Fetch the <em>example request input</em> of the named <em>BYOML Model</em>'
             ),
             'links': {
-                'doc': '#example-input',
-                'iodoc': 'example-input'
+                'doc': 'example-input'
             },
         },
         'predict': {
@@ -270,8 +244,7 @@ class ModelResource(WaylayResource):
                 'Execute the <em>predict</em> capability of the named <em>BYOML Model</em>'
             ),
             'links': {
-                'doc': '#predictions',
-                'iodoc': 'predictions'
+                'doc': 'predictions'
             },
         },
         'regress': {
@@ -282,8 +255,7 @@ class ModelResource(WaylayResource):
                 'Execute the <em>regress</em> capability of the named  <em>BYOML Model</em>'
             ),
             'links': {
-                'doc': '#predictions',
-                'iodoc': 'predictions'
+                'doc': 'predictions'
             },
         },
         'classify': {
@@ -294,18 +266,20 @@ class ModelResource(WaylayResource):
                 'Execute the <em>classification</em> capability of the named <em>BYOML Model</em>'
             ),
             'links': {
-                'doc': '#predictions',
-                'iodoc': 'predictions'
+                'doc': 'predictions'
             },
         },
         'remove': {
             'method': 'DELETE',
             'url': '/models/{}',
-            'decorators': DEFAULT_DECORATORS,
+            'decorators': [
+                byoml_server_timing_decorator,
+                byoml_exception_decorator,
+                return_body_decorator,
+            ],
             'description': 'Remove the named <em>BYOML Model</em>',
             'links': {
-                'doc': '#deleting-a-model',
-                'iodoc': 'deleting-a-model'
+                'doc': 'deleting-a-model'
             },
         },
     }
@@ -315,78 +289,64 @@ class ModelResource(WaylayResource):
         kwargs.pop('timeout', None)
         super().__init__(*args, timeout=DEFAULT_BYOML_MODEL_TIMEOUT, **kwargs)
 
-    def _get_files_to_zip(self, file_or_dir: PathLike) -> List[Tuple[str, str]]:
-        """Get the filenames to zip and the name it should have in the zipfile."""
-        file_or_dir = str(file_or_dir)
-        if not os.path.isdir(file_or_dir):
-            # single file
-            zip_file_name = os.path.basename(file_or_dir)
-            return [(file_or_dir, zip_file_name)]
-
-        file_names: List[Tuple[str, str]] = []
-        for root, _, files in os.walk(file_or_dir):
-            for file_name in files:
-                # the root will always contain the same suffix, which should not end up in the zip
-                zip_root = root[len(file_or_dir):]
-                zip_file_name = os.path.join(zip_root, file_name)
-
-                file_path = os.path.join(root, file_name)
-
-                file_names.append((file_path, zip_file_name))
-
-        return file_names
-
+    @contextmanager
     def _send_model_arguments(
         self, model_name: str, trained_model: Union[PathLike, ByomlModel],
-        framework: str = "sklearn", description: str = "",
-        work_dir: Optional[PathLike] = None
+        framework: str = "sklearn",
+        framework_version: Optional[str] = None,
+        metadata: Optional[Dict] = None,
+        work_dir: Optional[PathLike] = None,
+        **kwargs
     ):
         """Upload a binary model with given name, framework and description."""
-        if work_dir:
-            model_zip_buffer = self._save_model_in_dir(trained_model, work_dir, framework)
-        else:
-            with tempfile.TemporaryDirectory() as work_dir:
-                model_zip_buffer = self._save_model_in_dir(trained_model, work_dir, framework)
-        return {
-            'body': {
-                "name": model_name,
-                "framework": framework,
-                "description": description
-            },
-            'files': {
-                "file": ('model.zip', model_zip_buffer.getvalue())
-            },
-        }
+        with ModelZipArchiveBuilder(work_dir) as model_builder:
+            with model_builder.save_model_in_dir(trained_model, framework) as model_zip_buffer:
+                yield {
+                    'body': {
+                        "name": model_name,
+                        "framework": framework,
+                        "framework_version": framework_version,
+                        "metadata": json.dumps(metadata or {}),
+                    },
+                    'files': {
+                        "file": ('model.zip', model_zip_buffer.getvalue())
+                    },
+                    **kwargs
+                }
 
-    def _save_model_in_dir(
-        self, trained_model: Union[PathLike, ByomlModel], work_dir: PathLike, framework: str
+    @contextmanager
+    def _send_model_plug_arguments(
+        self, model_name: str,
+        trained_model: Union[PathLike, ByomlModel],
+        framework: str = "sklearn",
+        framework_version: Optional[str] = None,
+        metadata: Optional[Dict] = None,
+        requirements_file: Optional[PathLike] = None,
+        requirements: Optional[str] = None,
+        lib: Optional[PathLike] = None,
+        work_dir: Optional[PathLike] = None,
+        **kwargs
     ):
-        if not isinstance(trained_model, (str, os.PathLike)):
-            file_name = self._serialize_model(trained_model, work_dir, framework)
-            files = self._get_files_to_zip(file_name)
-        else:
-            files = self._get_files_to_zip(trained_model)
-
-        model_zip_buffer = BytesIO()
-        with ZipFile(model_zip_buffer, 'w') as zipper:
-            for file_name, zip_file_name in files:
-                zipper.write(file_name, zip_file_name)
-
-        return model_zip_buffer
-
-    def _serialize_model(
-        self, trained_model: ByomlModel, work_dir: PathLike, framework: str
-    ) -> PathLike:
-        assert_dir_exists(work_dir)
-
-        framework_function = SUPPORTED_FRAMEWORKS.get(framework, None)
-        if framework_function is not None:
-            return framework_function(work_dir, trained_model)
-
-        raise ByomlValidationError(
-            f'Passing a model instance is not supported for this `{framework}` model, '
-            'please provide the path to the saved model instead.'
-        )
+        runtime = self.service.framework.find_runtime(framework, framework_version)
+        with ModelPlugArchiveBuilder(work_dir) as builder:
+            builder.add_model_spec({
+                "name": model_name,
+                "version": "0.0.1",
+                "metadata": metadata,
+                "runtime": runtime['name']
+            })
+            builder.add_model(trained_model, runtime.get('framework', framework))
+            builder.add_requirements(requirements_file, requirements)
+            builder.add_lib(lib)
+            with builder.create_plug_tar_archive() as (tar_bytes, tar_size):
+                headers = kwargs.pop('headers', {})
+                headers['Content-Type'] = 'application/tar+gzip'
+                headers['Content-Length'] = str(tar_size)
+                yield {
+                    'body': tar_bytes,  # TODO or 'content': tar_bytes ??
+                    'headers': headers,
+                    **kwargs
+                }
 
     @suppress_header_decorator('Content-Type')
     def upload(
@@ -394,7 +354,12 @@ class ModelResource(WaylayResource):
         model_name: str,
         trained_model: Union[PathLike, ByomlModel],
         framework: str = "sklearn",
-        description: str = "",
+        framework_version: Optional[str] = None,
+        metadata: Optional[Dict] = None,
+        description: Optional[str] = None,
+        requirements_file: Optional[PathLike] = None,
+        requirements: Optional[str] = None,
+        lib: Optional[PathLike] = None,
         work_dir: Optional[PathLike] = None,
         **kwargs
     ) -> Any:
@@ -405,20 +370,55 @@ class ModelResource(WaylayResource):
             trained_model   The model object (will be serialised to an zip archive before upload),
                             or a file path to the serialized model file or folder.
             framework       One of the supported frameworks (default 'sklearn').
-            description     Description of the model.
+            framework_version
+                            A supported framework version, (default `None`,
+                            which selects the default server side)
+            metadata       User modifiable metadata. Standardised ones are:
+                            * 'description' A description of the model.
+                            * 'author' The attributed author of the model.
+                            * 'title' Used as display title in user interfaces.
+            description     Description of the model. (Deprecated: prefer `metadata.description`)
+            requirements_file
+                            A file containing custom python requirements (above that of the framework)
+            requirements    Custom python requirements (above those of the framework).
+                            Only one of 'requirements' and 'requirements_file' can be specified.
+            lib             A file or folder with custom packages that can be referred to in
+                            the requirements file as 'lib/<custom-requirement>'
             work_dir        Optional location of the working directory used to serialize the model.
                             If not specified, a temporary directory is used.
             (other args)    Passed onto the underlying 'POST /model/{name}' request
         """
-        return self._create(  # pylint: disable=no-member
-            **self._send_model_arguments(
+        if requirements or requirements_file or lib:
+            # use the new plug format upload (openfaas only)
+            with self._send_model_plug_arguments(
                 model_name, trained_model,
                 framework=framework,
-                description=description,
+                framework_version=framework_version,
+                metadata={
+                    'description': description,
+                    **(metadata or {})
+                },
+                requirements_file=requirements_file,
+                requirements=requirements,
+                lib=lib,
                 work_dir=work_dir,
-            ),
-            **kwargs
-        )
+                **kwargs
+            ) as arguments:
+                return self._create(**arguments)  # pylint: disable=no-member
+        else:
+            # legacy format (to be removed after move to openfaas byoml 1.4.0)
+            with self._send_model_arguments(
+                model_name, trained_model,
+                framework=framework,
+                framework_version=framework_version,
+                metadata={
+                    'description': description,
+                    **(metadata or {})
+                },
+                work_dir=work_dir,
+                **kwargs
+            ) as arguments:
+                return self._create(**arguments)  # pylint: disable=no-member
 
     @suppress_header_decorator('Content-Type')
     def replace(
@@ -426,7 +426,9 @@ class ModelResource(WaylayResource):
         model_name: str,
         trained_model: Union[PathLike, ByomlModel],
         framework: str = "sklearn",
-        description: str = "",
+        framework_version: Optional[str] = None,
+        metadata: Optional[Dict] = None,
+        description: Optional[str] = None,
         work_dir: Optional[PathLike] = None,
         **kwargs
     ) -> Any:
@@ -436,18 +438,30 @@ class ModelResource(WaylayResource):
             model_name      The name of the model.
             trained_model   The model object (will be serialised to an zip archive before upload).
             framework       One of the supported frameworks (default 'sklearn').
-            description     Description of the model.
+            framework_version
+                            A supported framework version, (default `None`,
+                            which selects the default server side)
+            metadata       User modifiable metadata. Standardised ones are:
+                            * 'description' A description of the model.
+                            * 'author' The attributed author of the model.
+                            * 'title' Used as display title in user interfaces.
+            description     Description of the model. (Deprecated: prefer `metadata.description`)
             work_dir        Optional location of the working directory used to serialize the model.
                             If not specified, a temporary directory is used.
             (other)         Passed onto the underlying 'PUT /model/{name}' request.
         """
-        return self._replace(   # pylint: disable=no-member
-            model_name,
-            **self._send_model_arguments(
+        with self._send_model_arguments(
                 model_name, trained_model,
                 framework=framework,
-                description=description,
+                framework_version=framework_version,
+                metadata={
+                    'description': description,
+                    **(metadata or {})
+                },
                 work_dir=work_dir,
-            ),
-            **kwargs
-        )
+                **kwargs
+        ) as arguments:
+            return self._replace(   # pylint: disable=no-member
+                model_name,
+                **arguments,
+            )
