@@ -10,7 +10,7 @@ class ImportResource(WaylayResource):
     """REST Resource for the import process entity of the etl service."""
 
     link_roots = {
-        'iodoc': '${iodoc_url}/features/etl/?id='
+        'doc': '${doc_url}/features/etl/?id='
     }
 
     actions = {
@@ -20,7 +20,7 @@ class ImportResource(WaylayResource):
             'decorators': DEFAULT_DECORATORS,
             'description': "Initiates an etl import process as specified in the request body.",
             'links': {
-                'iodoc': 'etl-import-service',
+                'doc': 'etl-import-service',
             }
         },
         'get': {
@@ -29,7 +29,7 @@ class ImportResource(WaylayResource):
             'decorators': DEFAULT_DECORATORS,
             'description': "Retrieves the last or active etl import process.",
             'links': {
-                'iodoc': 'etl-import-service',
+                'doc': 'etl-import-service',
             }
         }
     }
