@@ -9,7 +9,7 @@ class AboutResource(WaylayResource):
     """Static information about version."""
 
     link_roots = {
-        'iodoc': '${iodoc_url}/api/storage/?id=',
+        'doc': '${doc_url}/api/storage/?id=',
         'apidoc':  '${root_url}/docs#/status/',
         'openapi': '${root_url}/openapi.json'
     }
@@ -21,7 +21,7 @@ class AboutResource(WaylayResource):
             'decorators': DEFAULT_DECORATORS,
             'description': 'Application version',
             'links': {
-                'iodoc': 'version',
+                'doc': 'version',
                 'apidoc': 'version__get',
                 'openapi': f"#/paths/{quote('/')}/get",
             },
@@ -32,7 +32,7 @@ class AboutResource(WaylayResource):
             'decorators': DEFAULT_DECORATORS,
             'description': 'Validation and statistics on the buckets and policies for this tenant.',
             'links': {
-                'iodoc': 'tenant-status',
+                'doc': 'tenant-status',
                 'apidoc': 'status_status_get',
                 'openapi': f"#/paths/{quote('/status')}/get",
             },
