@@ -8,7 +8,7 @@ class BucketResource(WaylayResource):
     """REST Resource for the 'bucket' entity of the 'storage' service."""
 
     link_roots = {
-        'iodoc': '${iodoc_url}/api/storage/?id=',
+        'doc': '${doc_url}/api/storage/?id=',
         'apidoc':  '${root_url}/docs#/storage/',
         'openapi': '${root_url}/openapi.json'
     }
@@ -21,7 +21,7 @@ class BucketResource(WaylayResource):
             ],
             'description': 'List available bucket aliases',
             'links': {
-                'iodoc': 'list-bucket',
+                'doc': 'list-bucket',
                 'apidoc': 'list_buckets_bucket__get',
                 'openapi': f"#/paths/{quote('/bucket')}/get"
             }
@@ -33,7 +33,7 @@ class BucketResource(WaylayResource):
             ],
             'description': 'Get metadata for a specific bucket alias',
             'links': {
-                'iodoc': 'get-bucket',
+                'doc': 'get-bucket',
                 'apidoc': 'get_bucket_bucket__bucket_name__get',
                 'openapi': f"#/paths/{quote('/bucket/{bucket_name}')}/get"
             }
