@@ -6,6 +6,7 @@ from waylay.service.storage import StorageService
 from waylay.service.etl import ETLService
 from waylay.service.api import ApiService
 from waylay.service.analytics import AnalyticsService
+from waylay.service.data import DataService
 
 from .tool import TimeSeriesETLTool
 
@@ -21,10 +22,11 @@ class TimeSeriesService(WaylayService):
         """Configure endpoints and authentication with given config."""
         self.config = config
         self.etl_tool = TimeSeriesETLTool(
-            context.require(StorageService),
-            context.require(ETLService),
-            context.require(ApiService),
-            context.require(AnalyticsService),
+            storage_service=context.require(StorageService),
+            etl_service=context.require(ETLService),
+            api_service=context.require(ApiService),
+            data_service=context.require(DataService),
+            analytics_service=context.require(AnalyticsService),
         )
         return self.reconfigure()
 
