@@ -1,9 +1,12 @@
 """ETL import tooling."""
 __docformat__ = "google"
 
-from typing import Any, Optional, List, Dict, Iterator, Union
+from typing import (
+    Any, Optional, List,
+    Dict, Iterator, Union
+)
 from enum import Enum
-from dataclasses import dataclass, asdict
+from dataclasses import dataclass, asdict, replace
 from contextlib import contextmanager
 from pathlib import PosixPath
 import logging
@@ -14,10 +17,13 @@ from waylay.exceptions import WaylayError
 from waylay.service.storage import StorageService
 from waylay.service.etl import ETLService
 from waylay.service.api import ApiService
+from waylay.service.data import DataService
 from waylay.service.analytics import AnalyticsService
 
 from .parser.model import (
     SeriesInput,
+    CSVOutput,
+    ArchiveType,
     SeriesSettings,
     WaylayETLSeriesImport,
     ETLFile,
@@ -36,6 +42,8 @@ from .parser import (
     list_resources,
 )
 
+from .parser import export_series
+
 LOG = logging.getLogger(__name__)
 
 ETL_IMPORT_TIMESERIES_FILE_SUFFIX = '-timeseries.csv.gz'
@@ -183,7 +191,7 @@ class ETLImportJob:
 
 
 class TimeSeriesETLTool:
-    """Client tool to prepare and upload timeseries data.
+    """Client tool to prepare and upload or download timeseries data.
 
     Attributes:
         storage_service:
@@ -199,6 +207,7 @@ class TimeSeriesETLTool:
         storage_service: StorageService,
         etl_service: ETLService,
         api_service: ApiService,
+        data_service: DataService,
         analytics_service: AnalyticsService,
         temp_dir: Optional[PathLike] = None
     ):
@@ -206,12 +215,13 @@ class TimeSeriesETLTool:
         self.storage_service = storage_service
         self.etl_service = etl_service
         self.api_service = api_service
+        self.data_service = data_service
         self.analytics_service = analytics_service
         self.temp_dir = temp_dir
 
     def prepare_import(
         self,
-        series: SeriesInput,
+        *series: SeriesInput,
         name: Optional[str] = None,
         temp_dir: Optional[PathLike] = None,
         settings: Optional[SeriesSettings] = None,
@@ -239,13 +249,14 @@ class TimeSeriesETLTool:
                 Any attribute of a `SeriesSettings` object. See `settings` above.
         """
         etl_file = ETLFile(temp_dir or self.temp_dir, name)
-        etl_import = prepare_etl_import(series, etl_file, settings, **settings_args)
+        etl_import = prepare_etl_import(*series, import_file=etl_file, settings=settings, **settings_args)
         return create_etl_import(etl_import, progress=progress)
 
     def initiate_import(
         self,
         etl_import: WaylayETLSeriesImport,
-        resource_update_level: ResourceUpdateLevel = ResourceUpdateLevel.ID
+        resource_update_level: ResourceUpdateLevel = ResourceUpdateLevel.ID,
+        progress: bool = False
     ) -> WaylayETLSeriesImport:
         """Upload a prepared timeseries file to the etl-import ingestion bucket.
 
@@ -259,6 +270,7 @@ class TimeSeriesETLTool:
             etl_import.storage_object_name,
             from_file=etl_import.import_file.path,
             content_type='application/gzip',
+            progress=progress
         )
         if resource_update_level:
             try:
@@ -570,3 +582,27 @@ class TimeSeriesETLTool:
         }
         query.update(query_params)
         return self.analytics_service.query.replace(query_name, body={'query': query})
+
+    def export_series_as_csv(
+        self,
+        output: CSVOutput,
+        archive_type: Optional[str] = None,
+        progress: bool = True,
+        settings: Optional[SeriesSettings] = None,
+        timestamp_column: Optional[str] = 'timestamp',
+        **settings_args: Any,
+    ):
+        """Export a number of series from the times series database."""
+        export_settings = settings or SeriesSettings()
+        export_settings = replace(export_settings, **settings_args)
+        # set timestamp column (default `timestamp`) unless explicitely set to None
+        if timestamp_column:
+            export_settings.timestamp_column = timestamp_column
+        series_provider = export_series.PagingExportSeriesProvider(export_settings, self.data_service.series)
+        return export_series.export_csv(
+            output,
+            export_settings,
+            series_provider,
+            ArchiveType.lookup(archive_type),
+            progress,
+        )
